@@ -55,7 +55,7 @@ class GDNCollectionTargetConnector(C8Connector):
             ConfigProperty('region', 'Region URL', ConfigAttributeType.STRING, True, False,
                            description="Fully qualified region URL.",
                            placeholder_value='api-sample-ap-west.eng.macrometa.io'),
-            ConfigProperty('api_key', 'API Key', ConfigAttributeType.STRING, True, False,
+            ConfigProperty('api_key', 'API Key', ConfigAttributeType.PASSWORD, True, False,
                            description="API key.",
                            placeholder_value='my_apikey'),
             ConfigProperty('fabric', 'Fabric', ConfigAttributeType.STRING, True, False,
@@ -66,15 +66,15 @@ class GDNCollectionTargetConnector(C8Connector):
                            placeholder_value='my_collection'),
             ConfigProperty('batch_size_rows', 'Batch Size', ConfigAttributeType.INT, False, False,
                            description='Maximum number of rows inserted per batch.',
-                           default_value='50'),
+                           default_value='100'),
             ConfigProperty('batch_flush_interval', 'Batch Flush Interval (Seconds)',
                            ConfigAttributeType.INT, False, False,
                            description='Time between batch flush executions.',
-                           default_value='60'),
+                           default_value='10'),
             ConfigProperty('batch_flush_min_time_gap', 'Batch Flush Minimum Time Gap (Seconds)',
                            ConfigAttributeType.INT, False, False,
                            description='Minimum time gap between two batch flush tasks.',
-                           default_value='60'),
+                           default_value='10'),
         ]
 
     def capabilities(self) -> list[str]:
