@@ -11,7 +11,7 @@ import requests
 import sys
 import time
 from asyncio import AbstractEventLoop
-from datetime import datetime
+from datetime import datetime, timezone
 from threading import Thread, Lock
 
 import jsonschema
@@ -24,9 +24,9 @@ from singer import get_logger
 
 logger = get_logger('macrometa_target_collection')
 
-DEFAULT_BATCH_SIZE_ROWS = 50
-DEFAULT_BATCH_FLUSH_INTERVAL = 60
-DEFAULT_MIN_BATCH_FLUSH_TIME_GAP = 60
+DEFAULT_BATCH_SIZE_ROWS = 100
+DEFAULT_BATCH_FLUSH_INTERVAL = 10
+DEFAULT_MIN_BATCH_FLUSH_TIME_GAP = 10
 
 # Prometheus metrics
 ingested_bytes = Counter('ingested_bytes', 'Total number of bytes ingested', ['region', 'tenant', 'fabric', 'workflow'])
@@ -40,9 +40,12 @@ tenant_label = os.getenv("GDN_TENANT", "NA")
 fabric_label = os.getenv("GDN_FABRIC", "NA")
 workflow_label = os.getenv("WORKFLOW_UUID", "NA")
 metric_service_url = os.getenv("METRIC_SERVICE_API")
+is_metrics_enabled = os.getenv("MACROMETA_TARGET_COLLECTION_IS_METRICS_ENABLED", 'False')
 
 # Start the Prometheus HTTP server for exposing metrics
-start_http_server(8000)
+if is_metrics_enabled.lower() == 'true':
+    logger.info("Target is starting the metrics server.")
+    start_http_server(8000)
 
 class RecordBatch:
     """Class wrapping the record batch in order to make it thread safe."""
@@ -51,7 +54,7 @@ class RecordBatch:
         self._list = list()
         self._lock = Lock()
         self.interval = config.get('batch_flush_interval', DEFAULT_BATCH_FLUSH_INTERVAL)
-        self.last_executed_time = datetime.now()
+        self.last_executed_time = datetime.now(timezone.utc)
         self.min_time_gap = config.get('batch_flush_min_time_gap', DEFAULT_MIN_BATCH_FLUSH_TIME_GAP)
         self.max_batch_size = config.get('batch_size_rows', DEFAULT_BATCH_SIZE_ROWS)
 
@@ -86,7 +89,9 @@ def try_upsert(collection: StandardCollection, record_batch: RecordBatch, force=
     if record_batch.length() >= record_batch.max_batch_size or force:
         to_insert = record_batch.flush()
         to_update = []
-        for i, r in enumerate(collection.insert_many(to_insert)):
+        records_array = remove_time_extracted(to_insert)
+
+        for i, r in enumerate(collection.insert_many(records_array)):
             if type(r) is DocumentInsertError:
                 to_update.append(to_insert[i])
             else:
@@ -94,20 +99,25 @@ def try_upsert(collection: StandardCollection, record_batch: RecordBatch, force=
                 ingested_documents.labels(region_label, tenant_label, fabric_label, workflow_label).inc()
                 ingested_bytes.labels(region_label, tenant_label, fabric_label, workflow_label).inc(len(json.dumps(r)))
                 # Update ingest_lag metric
-                diff = datetime.utcnow() - datetime.strptime(to_insert[i]["time_extracted"], "%Y-%m-%dT%H:%M:%S.%fZ")
+                diff = datetime.now(timezone.utc) - ensure_datetime(to_insert[i]["time_extracted"])
                 ingest_lag.labels(region_label, tenant_label, fabric_label, workflow_label).set(diff.total_seconds())
 
         if len(to_update) > 0:
-            for i, r in enumerate(collection.update_many(to_update)):
+            records_array = remove_time_extracted(to_update)
+            for i, r in enumerate(collection.update_many(records_array)):
                 if type(r) is DocumentInsertError:
                     # Increment ingest_errors metric
                     ingest_errors.labels(region_label, tenant_label, fabric_label, workflow_label).inc()
                     print(f'Failed to insert/update record: {to_update[i]}. {r}')
                 else:
                     # Update ingest_lag metric
-                    diff = datetime.utcnow() - datetime.strptime(to_update[i]["time_extracted"], "%Y-%m-%dT%H:%M:%S.%fZ")
+                    diff = datetime.now(timezone.utc) - ensure_datetime(to_update[i]["time_extracted"])
                     ingest_lag.labels(region_label, tenant_label, fabric_label, workflow_label).set(diff.total_seconds())
-        record_batch.last_executed_time = datetime.now()
+        record_batch.last_executed_time = datetime.now(timezone.utc)
+
+
+def remove_time_extracted(records):
+    return [{k: v for k, v in record.items() if k != 'time_extracted'} for record in records]
 
 
 def try_delete(collection: StandardCollection, _key: str):
@@ -152,10 +162,10 @@ def persist_messages(collection: StandardCollection, messages: io.TextIOWrapper,
 
             try:
                 rec = o['record']
-                if o["time_extracted"]:
-                    rec["time_extracted"] = o["time_extracted"]
+                if 'time_extracted' in o:
+                    rec["time_extracted"] = ensure_datetime(o["time_extracted"])
                 else:
-                    rec["time_extracted"] = record_batch.last_executed_time
+                    rec["time_extracted"] = ensure_datetime(record_batch.last_executed_time)
                 try:
                     kps = key_properties.get(stream)
                     _key = None
@@ -211,6 +221,7 @@ def persist_messages(collection: StandardCollection, messages: io.TextIOWrapper,
         elif message_type == 'STATE':
             logger.debug('Setting state to {}'.format(o['value']))
             state = o['value']
+            emit_state(state)
         elif message_type == 'SCHEMA':
             stream = o['stream']
             schemas[stream] = o['schema']
@@ -226,6 +237,17 @@ def persist_messages(collection: StandardCollection, messages: io.TextIOWrapper,
     return state
 
 
+def ensure_datetime(time_extracted):
+    if isinstance(time_extracted, str):
+        time_extracted = datetime.strptime(time_extracted, "%Y-%m-%dT%H:%M:%S.%fZ")
+        # Make the datetime object timezone-aware by setting it to UTC timezone
+        time_extracted = time_extracted.replace(tzinfo=timezone.utc)
+    elif time_extracted.tzinfo is None:
+        # If the datetime object is timezone-naive, set it to UTC timezone
+        time_extracted = time_extracted.replace(tzinfo=timezone.utc)
+    return time_extracted
+
+
 def setup_batch_task(collection: StandardCollection, record_batch: RecordBatch) -> AbstractEventLoop:
     event_loop = asyncio.new_event_loop()
     Thread(target=start_background_loop, args=(event_loop,), daemon=True).start()
@@ -236,7 +258,7 @@ def setup_batch_task(collection: StandardCollection, record_batch: RecordBatch)
 async def process_batch(collection: StandardCollection, record_batch: RecordBatch) -> None:
     while True:
         await asyncio.sleep(record_batch.interval)
-        timedelta = datetime.now() - record_batch.last_executed_time
+        timedelta = datetime.now(timezone.utc) - ensure_datetime(record_batch.last_executed_time)
         if timedelta.total_seconds() >= record_batch.min_time_gap:
             # if batch has records that need to be processed but haven't reached batch size then process them.
             try_upsert(collection, record_batch, force=True)
@@ -280,9 +302,16 @@ def main():
     # There can still be records in the `record_batch` which is not processed,
     # So, we have to force process it one last time before the workflow terminates.
     try_upsert(collection, record_batch, force=True)
+
+    # Wait for Prometheus to scrape the metrics
+    logger.info("Waiting for metrics scrape...")
+    while not is_scrape_complete(metric_service_url, f"{scrape_complete_flag._name}_total", f"workflow=\"{workflow_label}\""):
+        time.sleep(15)
+    logger.info("Metrics scrape complete. Exiting...")
+
     emit_state(state)
     event_loop.stop()
-    logger.info("Exiting normally...")
+    logger.info("Completing normally...")
 
 
 def is_scrape_complete(prometheus_url, metric_name, filter):
@@ -296,7 +325,3 @@ def is_scrape_complete(prometheus_url, metric_name, filter):
 
 if __name__ == '__main__':
     main()
-
-    # Wait for Prometheus to scrape the metrics
-    while not is_scrape_complete(metric_service_url, f"{scrape_complete_flag._name}_total", f"workflow=\"{workflow_label}\""):
-        time.sleep(15)
